@@ -17,7 +17,7 @@ def tokenize(seq):
     digit_seq = []
     for aa in seq:
         if aa not in AA_DICT:
-            # print(f'Unknown amino acid letter \"{aa}\"')
+            print(f'Unknown amino acid letter \"{aa}\"')
             return None
         digit_seq.append(AA_DICT[aa])
 
